#!/usr/bin/perl
use strict;
use warnings;

use XML::Simple;
use Data::Dumper;

#Argumentua: bertsolari kopurua
die("Programak argumentu bat behar du\n") if ($#ARGV != 0);

my $xml = new XML::Simple;
my $data = $xml->XMLin("corpusaAnalizatutaEzestOsoa.xml");

#////////////////////////////////////////////////////////////////
#BERTSOLARIEN BERTSO KOPURUAK GORDE
#////////////////////////////////////////////////////////////////

my %Bertso_kopurua;

foreach my $bertso (@{$data->{Bertsoa}})
{

    my $bertsolaria = $bertso->{Bertsolariak};

    $bertsolaria =~ tr/a-z/A-Z/;
    $bertsolaria =~ s/\s//g;
    $bertsolaria =~ s/,/-/g;
    
    if (ref($bertso->{Transkripzioa}->{lg}) ne 'ARRAY'){
        $Bertso_kopurua{$bertsolaria}++;
    }else{
        foreach my $lerroak (@{$bertso->{Transkripzioa}->{lg}}){
            $Bertso_kopurua{$bertsolaria}++;
        }
    }
}

#Bertsolariak  hash-ean atxikitutako balioen arabera ordenatu, handienetik txikienera
my @bertsolariOrdenatuak = sort ({$Bertso_kopurua{$b} <=> $Bertso_kopurua{$a}} keys(%Bertso_kopurua));


my $n = $ARGV[0];#bertsolari kopurua

#////////////////////////////////////////////////////////////////
#BERTSO GEHIENAK DITUZTEN n BERTSOLARIAK GORDE ETA PANTAILARATU
#////////////////////////////////////////////////////////////////


my $j = 0;
my @Bertsolariak2;

foreach my $b (@bertsolariOrdenatuak)
{
    if ($j < $n){
        if ($b =~ /\//){
            delete ($Bertso_kopurua{$b});
        }else{
            print "$b: $Bertso_kopurua{$b}\n";
            $Bertsolariak2[$j]=$b;
            $j++;
        }
    }else{
        #Gainontzeko bertsolariak hash-etik ezabatu
        delete ($Bertso_kopurua{$b});
        $j++;
    }
}


print "---------------------------------------------\n\n";
#////////////////////////////////////////////////////////////////
#CORPUSA SORTU
#////////////////////////////////////////////////////////////////


#Idazketarako helburu fitxategia ireki
open(FITXOUT, ">Corpus.xml") or die("Ezin da Corpus.xml sortu!\n");


#Corpus.xml fitxategiko edukia sortu
print (FITXOUT "<?xml version='1.0' encoding='ISO-8859-1'?>\n\n");

print (FITXOUT "<Bertsoak>\n\n");

my $k=1;
my $bertso_max = $Bertso_kopurua{$Bertsolariak2[$n-1]};
my (%Hitz_kop, %Bertso_kop);

foreach my $ber (@Bertsolariak2)
{
    print (FITXOUT "<Bertsolaria id=\"$k\">\n");
    print (FITXOUT "\t<Izena>$ber</Izena>\n");
    print (FITXOUT "\t<Transkripzioa>\n");
    
    $Hitz_kop{$ber} = 0;
    $Bertso_kop{$ber} = 0;
    
    foreach my $bertso (@{$data->{Bertsoa}})
    {
        
        my $bertsolaria = $bertso->{Bertsolariak};
        
        $bertsolaria =~ tr/a-z/A-Z/;
        $bertsolaria =~ s/\s//g;
        $bertsolaria =~ s/,/-/g;
        
        
        if ($bertsolaria eq $ber and $Bertso_kop{$bertsolaria}<$bertso_max){
            
            if (ref($bertso->{Transkripzioa}->{lg}) eq 'ARRAY'){
                foreach my $lerroak (@{$bertso->{Transkripzioa}->{lg}}){
                    $Bertso_kop{$bertsolaria}++;
                    print (FITXOUT "\t\t<lg>\n");
                    foreach my $lerro (@{$lerroak->{l}}){
                        print (FITXOUT "\t\t\t<l>\n");
                        if (ref($lerro->{w}) eq 'ARRAY'){
                            foreach my $hitz (@{$lerro->{w}}){
                                $Hitz_kop{$bertsolaria}++;
                                print (FITXOUT XMLout($hitz,RootName=>"w"));
                            }
                        }else{
                            $Hitz_kop{$bertsolaria}++;
                            print (FITXOUT XMLout($lerro->{w},RootName=>"w"));
                        }
                        print (FITXOUT "\t\t\t</l>\n");
                    }
                    print (FITXOUT "\t\t</lg>\n");
                    last if ($Bertso_kop{$bertsolaria}>=$bertso_max);
                }
            }else{
                $Bertso_kop{$bertsolaria}++;
                print (FITXOUT "\t\t<lg>\n");
                foreach my $lerro (@{$bertso->{Transkripzioa}->{lg}->{l}}){
                    print (FITXOUT "\t\t\t<l>\n");
                    if (ref($lerro->{w}) eq 'ARRAY'){
                        foreach my $hitz (@{$lerro->{w}}){
                            $Hitz_kop{$bertsolaria}++;
                            print (FITXOUT XMLout($hitz,RootName=>"w"));
                        }
                    }else{
                        $Hitz_kop{$bertsolaria}++;
                        print (FITXOUT XMLout($lerro->{w},RootName=>"w"));
                    }
                    print (FITXOUT "\t\t\t</l>\n");
                }
                print (FITXOUT "\t\t</lg>\n");
            }
        }
    }
        
    print (FITXOUT "\t</Transkripzioa>\n");
    print (FITXOUT "</Bertsolaria>\n\n\n\n\n");
    $k++;
}

print (FITXOUT "</Bertsoak>");

foreach my $b (@Bertsolariak2)
{
       print "$b: $Hitz_kop{$b} token / $Bertso_kop{$b} bertso\n";
}


#Fitxategia itxi
close(FITXOUT);
